/* Copyright (c) 2010, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

        .syntax unified


        .text

        /* Filter coefficients, and oscillator waveforms */
        .include    "ssb2700.inc"
        .include    "ssbaliasing.inc"

osci:               .word 1, 0, -1, 0
oscq:               .word 0, -1, 0, 1


        .bss

        /* Anti-aliasing filter. Required size:
         * (Number of taps = N_SSB_ALIASING) * (2 channels I/Q) * (sizeof(int32) = 4).
         * There's a copy of this ring buffer, so we need twice the size:
         * N_SSB_ALIASING*2*4*2 bytes.
         * Having a copy of the ring buffer allows to always read the complete buffer linearly!
         */
afilter32:          .space (N_SSB_ALIASING * 2 * 4 * 2), 0
        /* Write index into the anti-aliasing buffer. This is where the next sample goes. */
aidx:               .word 0


filter32:           .space (2 * N_SSB2700 * 4), 0  /* i and q samples interleaved words */
index:              .word 0
filtersum:          .space 4*4

        .text

        .thumb_func
        .global dsp_demodSsb48_asm
dsp_demodSsb48_asm:
        /*  Local variables:
         *  0:  Number of input samples
         */
        .set    varN, 0
        .set    varSamplePtr, 4
        .set    varMode, 8
        .set    varGainShift, 12
        .set    varOutputPtr, 16
        .set    varOutputN, 20
        .set    sizeLocals, 24

        push    {r4-r12,lr}
        sub     sp, #sizeLocals

        ldr     r1, [r0, #0]
        str     r1, [sp, #varSamplePtr]
        ldr     r1, [r0, #4]
        str     r1, [sp, #varN]
        ldr     r1, [r0, #8]
        str     r1, [sp, #varOutputPtr]
        add     r1, r0, #12
        str     r1, [sp, #varOutputN]
        ldr     r1, [r0, #16]
        str     r1, [sp, #varMode]
        ldr     r1, [r0, #20]
        mov     r0, #0x0600
        udiv    r1, r1, r0
        rsb     r1, r1, #20
        str     r1, [sp, #varGainShift]

_demodSsb48_sampleLoop:
        /* Get next i and q samples, and put into filter taps */
        ldr     r11, =afilter32             /* Calculate write pointer (=&afilter32[aidx]) */
        ldr     r10, =aidx
        ldr     r9, [r10]
        lsl     r0, r9, #3
        add     r11, r0

        ldr     r2, [sp, #varSamplePtr]
        ldrsh   r0, [r2], #2                /* Load i half-word sample */
        ldrsh   r1, [r2], #2                /* Load q half-word sample */
        str     r2, [sp, #varSamplePtr]

        stmia   r11, {r0, r1}               /* Store in ring buffer (filter tap) */
        add     r12, r11, #(N_SSB_ALIASING * 2 * 4)
        stmia   r12, {r0, r1}               /* Store a copy */

        add     r9, #1                      /* aidx++ */
        and     r9, #(N_SSB_ALIASING - 1)
        str     r9, [r10]

        /* Calculate filter */
        add     r11, #(2*4)                 /* Point to next (=oldest!) sample */
        ldr     r10, =ssbaliasing
        mov     r5, #0                      /* sum = 0 */
        mov     r6, #0

        .rept N_SSB_ALIASING/2
        ldmia   r11!, {r0, r1, r2, r3}      /* Fetch i/q of two samples */
        ldmia   r10!, {r7, r8}              /* Coefficients */
        smlal   r5, r6, r2, r7              /* sum += afilter[i].i * consti[m] */
        smlal   r5, r6, r1, r8              /* sum += afilter[i].q * constq[m] */
        .endr

        /* The value in 'sum' contains the sum of i and q branches, and therefore
         * this is the desired sideband as a real signal!
         */
        lsr     r5, #15                     /* Scale down to 16 significant bits */
        lsl     r0, r6, #(32-15)
        orr     r5, r5, r0                  /* least significant word of 64-bit right shift */

        /* Mix down to zero and build analytical signal */
        ldr     r8, =index
        ldr     r12, [r8]
        and     r2, r12, #(4-1)             /* Four oscillator samples per period (12 kHz @ fs=48 kHz) */
        ldr     r1, =osci
        ldr     r0, [r1, r2, lsl #2]
        mul     r6, r5, r0
        ldr     r1, =oscq
        ldr     r0, [r1, r2, lsl #2]
        mul     r7, r5, r0

        /* Put into filter taps */
        ldr     r11, =filter32
        str     r6, [r11, r12, lsl #3]
        add     r10, r11, #4
        str     r7, [r10, r12, lsl #3]

        /* Prepare filter loop */
        mov     r2, #6
        udiv    r0, r12, r2
        mul     r0, r0, r2
        sub     r2, r12, r0                 /* R2 = polyphase */
        mov     r1, #(8*(N_SSB2700/6))      /* 2*(int32)*(N/R) */
        mul     r0, r1, r2
        ldr     r9, =ssbfilter2700
        add     r9, r0                      /* Pointer to current polyphase branch */
        mov     r3, #N_SSB2700/6
        ldr     r0, =filtersum
        ldmia   r0, {r5-r8}                 /* Load intermediate sum for i and q branch */

_demodSsb48_tapLoop:
        ldr     r0,[r11, r12, lsl #3]       /* filter32[i].i */
        ldr     r1,[r9],#4                  /* ssbfilter[m].i */
        smlal   r5, r6, r0, r1              /* sumi += tap * coeff */
        ldr     r0,[r10, r12, lsl #3]       /* filter32[i].q */
        ldr     r1,[r9],#4                  /* ssbfilter[m].q */
        smlal   r7, r8, r0, r1              /* sumq += tap * coeff */

        subs    r12, #6                     /* i -= R */
        it      mi
        addmi   r12, #N_SSB2700             /* modulo N_SSB2700 */

        subs    r3, r3, #1
        bne     _demodSsb48_tapLoop

        /* Check if there's a new sample (last polyphase branch) */
        cmp     r2, #(6-1)
        bne     _demodSsb48_noNewSample

        /* Select sideband */
        ldr     r0, [sp, #varMode]
        movs    r0, r0
        beq     _demodSsb48_usb
        subs    r5, r5, r7
        sbc     r6, r6, r8                  /* sumi - sumq (USB) */
        b       _demodSsb48_mode_end
_demodSsb48_usb:
        adds    r5, r5, r7
        adc     r6, r6, r8                  /* sumi + sumq (LSB) */
_demodSsb48_mode_end:


        /* Apply gain shift. Keep only lower 32-bit word. */
        ldr     r0, [sp, #varGainShift]
        lsr     r5, r5, r0
        rsb     r1, r0, #32
        lsl     r0, r6, r1
        orr     r5, r5, r0                  /* least significant word of 64-bit right shift */

        /* Saturate */
        ssat    r5, #16, r5

        /* Store result */
        ldr     r0, [sp, #varOutputPtr]
        strh    r5, [r0], #2
        str     r0, [sp, #varOutputPtr]

        ldr    r0, [sp, #varOutputN]
        ldr    r1, [r0]
        add    r1, #1
        str    r1, [r0]

        mov     r5, #0
        mov     r6, #0
        mov     r7, #0
        mov     r8, #0

_demodSsb48_noNewSample:
        /* Store intermediate filter sum */
        ldr     r0, =filtersum
        stmia   r0, {r5-r8}

        /* Index++ */
        ldr     r0, =index
        ldr     r12, [r0]
        add     r12, #1
        cmp     r12, #N_SSB2700
        it      hs
        movhs   r12, #0
        str     r12, [r0]

        /* Check if all input samples have been processed */
        ldr     r0, [sp, #varN]
        subs    r0, #1
        str     r0, [sp, #varN]
        bne     _demodSsb48_sampleLoop

        add     sp, #sizeLocals
        pop     {r4-r12,lr}

        bx      lr

        .end
