/* Copyright (c) 2010, DF9DQ
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions
 * and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 * Neither the name of the author nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

        .syntax unified


        .text

        /* Filter coefficients, and oscillator waveforms */
        .include    "am6500.inc"
        .include    "amaliasing.inc"

        /* 12-kHz oscillator */
osci:               .word 1, 0, -1, 0
oscq:               .word 0, -1, 0, 1


        .section .ram3bss, "w"

        /* Anti-aliasing filter. Required size:
         * (Number of taps = N_AM_ALIASING) * (2 channels I/Q) * (sizeof(int32) = 4).
         * There's a copy of this ring buffer, so we need twice the size:
         * N_AM_ALIASING*2*4*2 bytes.
         * Having a copy of the ring buffer allows to always read the complete buffer linearly!
         */
afilter32:          .space (N_AM_ALIASING * 2 * 4 * 2), 0
        /* Write index into the anti-aliasing buffer. This is where the next sample goes. */
aidx:               .word 0


filter32:           .space (2 * N_AM6500 * 4), 0  /* i and q samples interleaved words */
index:              .word 0
filtersum:          .space 4*4

        .set    DCFILTER_LENGTH, 128
dcFilter:           .space (4 * DCFILTER_LENGTH)
dcIndex:            .word 0
dcPrevious:         .word 0

        .text


accu1               .req    r5


        .thumb_func
        .global dsp_demodAm48
dsp_demodAm48:
        /*  Local variables:
         *  0:  Number of input samples
         */
        .set    varN, 0
        .set    varSamplePtr, 4
        .set    varMode, 8
        .set    varGainShift, 12
        .set    varOutputPtr, 16
        .set    varOutputN, 20
        .set    varRssiPtr, 24
        .set    sizeLocals, 28

        push    {r4-r12,lr}
        sub     sp, #sizeLocals

        ldr     r1, [r0, #0]
        str     r1, [sp, #varSamplePtr]
        ldr     r1, [r0, #4]
        str     r1, [sp, #varN]
        ldr     r1, [r0, #8]
        str     r1, [sp, #varOutputPtr]
        add     r1, r0, #12
        str     r1, [sp, #varOutputN]
        ldr     r1, [r0, #16]
        str     r1, [sp, #varMode]
        ldr     r1, [r0, #20]
        mov     r2, #0x0600
        udiv    r1, r1, r2
        rsb     r1, r1, #10
        str     r1, [sp, #varGainShift]
        add     r1, r0, #24
        str     r1, [sp, #varRssiPtr]

_demodAm48_sampleLoop:
        /* Get next i and q samples, and put into filter taps */
        ldr     r11, =afilter32             /* Calculate write pointer (=&afilter32[aidx]) */
        ldr     r10, =aidx
        ldr     r9, [r10]
        lsl     r0, r9, #3
        add     r11, r0

        ldr     r2, [sp, #varSamplePtr]
        ldrsh   r0, [r2], #2                /* Load i half-word sample */
        ldrsh   r1, [r2], #2                /* Load q half-word sample */
        str     r2, [sp, #varSamplePtr]

        stmia   r11, {r0, r1}               /* Store in ring buffer (filter tap) */
        add     r12, r11, #(N_AM_ALIASING * 2 * 4)
        stmia   r12, {r0, r1}               /* Store a copy */

        add     r9, #1                      /* aidx++ */
        and     r9, #(N_AM_ALIASING - 1)
        str     r9, [r10]

        /* Calculate filter */
        add     r11, #(2*4)                 /* Point to next (=oldest!) sample */
        ldr     r10, =amaliasing
        mov     accu1, #0                   /* sum = 0 */

        .rept N_AM_ALIASING/2
        ldmia   r11!, {r0, r1, r2, r3}      /* Fetch i/q of two samples */
        ldmia   r10!, {r7, r8}              /* Coefficients */
        mla     accu1, r2, r7, accu1        /* sum += afilter[i].i * consti[m] */
        mla     accu1, r1, r8, accu1        /* sum += afilter[i].q * constq[m] */
        .endr

        /* The value in 'sum' contains the sum of i and q branches, and therefore
         * this is the desired sideband as a real signal!
         */
        asr     accu1, #15                  /* Scale down to 16 significant bits */

        /* Mix down to zero and build analytical signal */
        ldr     r8, =index
        ldr     r12, [r8]
        and     r2, r12, #(4-1)             /* Four oscillator samples per period (12 kHz @ fs=48 kHz) */
        ldr     r1, =osci
        ldr     r0, [r1, r2, lsl #2]
        mul     r6, accu1, r0
        ldr     r1, =oscq
        ldr     r0, [r1, r2, lsl #2]
        mul     r7, accu1, r0

        /* Put into filter taps */
        ldr     r11, =filter32
        str     r6, [r11, r12, lsl #3]
        add     r10, r11, #4
        str     r7, [r10, r12, lsl #3]

        /* Prepare filter loop */
        mov     r2, #6
        udiv    r0, r12, r2
        mul     r0, r0, r2
        sub     r2, r12, r0                 /* R2 = polyphase */
        mov     r1, #(4*(N_AM6500/6))       /* 1*(int32)*(N/R) */
        mul     r0, r1, r2
        ldr     r9, =amfilter6500
        add     r9, r0                      /* Pointer to current polyphase branch */
        mov     r3, #N_AM6500/6
        ldr     r0, =filtersum
        ldmia   r0, {r5-r8}                 /* Load intermediate sum for i and q branch */

_demodAm48_tapLoop:
        ldr     r0,[r11, r12, lsl #3]       /* filter32[i].i */
        ldr     r1,[r9],#4                  /* amfilter[m] */
        smlal   r5, r6, r0, r1              /* sumi += tap * coeff */
        ldr     r0,[r10, r12, lsl #3]       /* filter32[i].q */
        smlal   r7, r8, r0, r1              /* sumq += tap * coeff */

        subs    r12, #6                     /* i -= R */
        it      mi
        addmi   r12, #N_AM6500              /* modulo N_AM6500 */

        subs    r3, r3, #1
        bne     _demodAm48_tapLoop

        /* Check if there's a new sample (last polyphase branch) */
        cmp     r2, #(6-1)
        bne     _demodAm48_noNewSample

        /* NF = sqrt(i^2 + q^2) - 1 */
        lsr     r5, r5, #22
        lsl     r0, r6, #(32-22)
        orr     r5, r5, r0                  /* least significant word of 64-bit right shift */
        lsr     r7, r7, #22
        lsl     r0, r8, #(32-22)
        orr     r7, r7, r0                  /* least significant word of 64-bit right shift */

        mul     r0, r5, r5                  /* i^2 */
        mla     r0, r7, r7, r0              /* + q^2 */

        clz     r1, r0                      /* First guess */
        rsb     r1, r1, #32
        lsr     r1, r1, #1
        lsr     r2, r0, r1

        cbz     r2, _demodAm48_noSqrt
        udiv    r1, r0, r2
        add     r2, r1
        lsr     r2, #1
        cbz     r2, _demodAm48_noSqrt
        udiv    r1, r0, r2
        add     r2, r1
        lsr     r2, #1
        cbz     r2, _demodAm48_noSqrt
        udiv    r1, r0, r2
        add     r2, r1
        lsr     r2, #1
_demodAm48_noSqrt:

        /* Moving average filter */
        ldr     r0, =dcFilter
        ldr     r1, =dcIndex
        ldr     r3, [r1]
        ldr     r4, [r0, r3, lsl #2]
        str     r2, [r0, r3, lsl #2]
        add     r3, #1
        cmp     r3, #DCFILTER_LENGTH
        it      eq
        moveq   r3, #0
        str     r3, [r1]

        sub     r4, r2, r4
        ldr     r0, =dcPrevious
        ldr     r5, [r0]
        add     r4, r4, r5
        str     r4, [r0]
        mov     r0, #DCFILTER_LENGTH
        udiv    r4, r4, r0

  clz r0, r4
  rsb r0, r0, #23
  mov r1, #6
  mul r0, r0, r1
  ldr r1, [sp, #varRssiPtr]
  str r0, [r1]



        sub     r5, r2, r4              /* - DC offset */

  mov r0,#32000  /* NF-Amplitude renormieren */
  cmp r4, #10
  it lt
  movlt r4,#10
  udiv r0,r0,r4
  mul r5,r5,r0

        /* Apply gain shift. Keep only lower 32-bit word. */
        ldr     r1, [sp, #varGainShift]
        asr     r5, r5, r1

        /* Saturate */
        ssat    r5, #16, r5

        /* Store result */
        ldr     r1, [sp, #varOutputPtr]
        strh    r5, [r1], #2
        str     r1, [sp, #varOutputPtr]

        ldr    r0, [sp, #varOutputN]
        ldr    r1, [r0]
        add    r1, #1
        str    r1, [r0]

        mov     r5, #0
        mov     r6, #0
        mov     r7, #0
        mov     r8, #0

_demodAm48_noNewSample:
        /* Store intermediate filter sum */
        ldr     r0, =filtersum
        stmia   r0, {r5-r8}

        /* Index++ */
        ldr     r0, =index
        ldr     r12, [r0]
        add     r12, #1
        cmp     r12, #N_AM6500
        it      hs
        movhs   r12, #0
        str     r12, [r0]

        /* Check if all input samples have been processed */
        ldr     r0, [sp, #varN]
        subs    r0, #1
        str     r0, [sp, #varN]
        bne     _demodAm48_sampleLoop

        add     sp, #sizeLocals
        pop     {r4-r12,lr}

        bx      lr

        .end
