/* USB device library for LPC1700 Cortex-M3 microcontrollers
 *
 * Copyright 2010 DF9DQ
 *
 * This file is part of the FiFi-SDR project <http://www.ov-lennestadt.de/fifisdr>
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Boot1700-USB.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Contains the application specific descriptors and strings */

#include <stdlib.h>

#include "usbdesc.h"
#include "init.h"


/* Array containing all descriptors in sequence:
 * Device, Configuration, Interface, Endpoint (2x)
 */
const uint8_t usb_descriptors[] =
{
    /* Device Descriptor */
    USBD_SIZE_DEVICE,               /* bLength                                  */
    USBD_DEVICE,                    /* bDescriptorType                          */
    LE16(0x0110),                   /* bcdUSB               1.10                */
    USBC_BASE,                      /* bDeviceClass         BASE                */
    0,                              /* bDeviceSubClass      (none)              */
    0,                              /* bDeviceProtocol      (none)              */
    64,                             /* bMaxPacketSize0      64                  */
    LE16(0x16C0),                   /* idVendor             0x16C0              */
    LE16(0x05DC),                   /* idProduct            0x05DC              */
    LE16(0x0100),                   /* bcdDevice            1.00                */
    1,                              /* iManufacturer                            */
    2,                              /* iProduct                                 */
    3,                              /* iSerialNumber                            */
    1,                              /* bNumConfigurations   1                   */

    /* Configuration Descriptor */
    USBD_SIZE_CONFIGURATION,        /* bLength                                  */
    USBD_CONFIGURATION,             /* bDescriptorType                          */
    LE16(                           /* wTotalLength                             */
        USBD_SIZE_CONFIGURATION +

        USBD_SIZE_INTERFACE +       /* Softrock                                 */

        USBD_SIZE_INTERFACE +       /* Soundcard IQ                             */
            USBD_SIZE_AUDIO_ST_HEADER(1) +
            USBD_SIZE_AUDIO_ST_INPUT_TERMINAL +
            USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL +
            USBD_SIZE_AUDIO_ST_FEATURE_UNIT(2,1) +
        USBD_SIZE_INTERFACE +
        USBD_SIZE_INTERFACE +
            7 +
            14 +
            USBD_SIZE_ENDPOINT +
            7 +

        USBD_SIZE_INTERFACE +       /* Soundcard Demodulator                    */
            USBD_SIZE_AUDIO_ST_HEADER(1) +
            USBD_SIZE_AUDIO_ST_INPUT_TERMINAL +
            USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL +
            USBD_SIZE_AUDIO_ST_FEATURE_UNIT(1,1) +
        USBD_SIZE_INTERFACE +
        USBD_SIZE_INTERFACE +
            7 +
            11 +
            USBD_SIZE_ENDPOINT +
            7 +

        0
      ),
    5,                              /* bNumInterfaces       5                   */
    1,                              /* bConfigurationValue  1                   */
    0,                              /* iConfiguration       (none)              */
    0x80,                           /* bmAttributes         bus-powered         */
    250/2,                          /* bMaxPower            250 mA              */

    /* --- Softrock Emulation --- */

    /* Softrock-40 Interface */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_SOFTROCK,         /* bInterfaceNumber                         */
    0,                              /* bAlternateSetting    0                   */
    0,                              /* bNumEndpoints        0                   */
    USBC_VENDOR_SPECIFIC,           /* bInterfaceClass                          */
    0,                              /* bInterfaceSubClass                       */
    0,                              /* bInterfaceProtocol                       */
    6,                              /* iInterface           "SoftRock SDR"      */

    /* --- Soundcard IQ --- */

    /* Standard AudioControl Interface */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOC_IQ,        /* bInterfaceNumber                         */
    0,                              /* bAlternateSetting    0                   */
    0,                              /* bNumEndpoints        0                   */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOCONTROL,       /* bInterfaceSubClass   AUDIOCONTROL        */
    0,                              /* bInterfaceProtocol                       */
    10,                             /* iInterface           "FiFiSDR Soundcard" */

    /* Class-Specific AC Interface Header */
    USBD_SIZE_AUDIO_ST_HEADER(1),
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_HEADER,           /* bDescriptorSubtype   HEADER              */
    LE16(0x0100),                   /* bcdADC               1.0                 */
    LE16(                           /* wTotalLength                             */
        USBD_SIZE_AUDIO_ST_HEADER(1) +
        USBD_SIZE_AUDIO_ST_INPUT_TERMINAL +
        USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL +
        USBD_SIZE_AUDIO_ST_FEATURE_UNIT(2,1) +
        0
      ),
    1,                              /* bInCollection        1                   */
    USB_INTERFACE_AUDIOS_IQ,        /* baInterfaceNr(1)                         */

    /* Input Terminal */
    USBD_SIZE_AUDIO_ST_INPUT_TERMINAL,
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_INPUT_TERMINAL,   /* bDescriptorSubtype   INPUT_TERMINAL      */
    1,                              /* bTerminalID          ID=1                */
    LE16(0x0201),                   /* wTerminalType        LineIn              */
    0,                              /* bAssocTerminal       no association      */
    2,                              /* bNrChannels          2                   */
    LE16(0x0003),                   /* wChannelConfig       Front L+R           */
    0,                              /* iChannelNames                            */
    0,                              /* iTerminal            (no string)         */

    /* Output Terminal */
    USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL,
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_OUTPUT_TERMINAL,  /* bDescriptorSubtype   OUTPUT_TERMINAL     */
    3,                              /* bTerminalID          ID=3                */
    LE16(0x0101),                   /* wTerminalType        USB Streaming       */
    0,                              /* bAssocTerminal       no association      */
    USB_UNITID_IQ_FEATURE,          /* bSourceID            (IQ Feature)        */
    0,                              /* iTerminal            (none)              */

    /* Feature Unit */
    USBD_SIZE_AUDIO_ST_FEATURE_UNIT(2,1),
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_FEATURE_UNIT,     /* bDescriptorSubtype   FEATURE_UNIT        */
    USB_UNITID_IQ_FEATURE,          /* bUnitID                                  */
    1,                              /* bSourceID            Terminal #1         */
    1,                              /* bControlSize         n = 1 Byte/channel  */
    0x03,                           /* bmaControls(0)       Master: Mute+Volume */
    0x00,                           /* bmaControls(0)                           */
    0x00,                           /* bmaControls(0)                           */
    9,                              /* iFeature             "Preamp"            */

    /* Standard AudioStreaming Interface (alternate setting #0) */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOS_IQ,        /* bInterfaceNumber                         */
    0,                              /* bAlternateSetting    0                   */
    0,                              /* bNumEndPoints        no endpoints        */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOSTREAMING,     /* bInterfaceSubClass   AUDIOSTREAMING      */
    0,                              /* bInterfaceProtocol                       */
    5,                              /* iInterface           "UDA1361 Capture"   */

    /* Standard AudioStreaming Interface (alternate setting #1) */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOS_IQ,        /* bInterfaceNumber                         */
    1,                              /* bAlternateSetting    1                   */
    1,                              /* bNumEndPoints        1 data endpoint     */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOSTREAMING,     /* bInterfaceSubClass   AUDIOSTREAMING      */
    0,                              /* bInterfaceProtocol                       */
    5,                              /* iInterface           "UDA1361 Capture"   */

    /* Audio Stream Audio Class */
    7,
    USBD_CS_INTERFACE,
    1,                              /* bDescriptorSubtype   AS_GENERAL,         */
    3,                              /* bTerminalLink        #3                  */
    1,                              /* bDelay               1                   */
    LE16(0x0001),                   /* wFormatTag           PCM                 */

    /* Format Type Audio */
    14,
    USBD_CS_INTERFACE,
    2,                              /* bDescriptorSubtype   FORMAT_TYPE         */
    1,                              /* bFormatType          TYPE_I              */
    2,                              /* bNrChannels          2                   */
#if AUDIO_SAMPLES == 16
    2,                              /* bSubFrameSize        2                   */
    16,                             /* bBitResolution       16                  */
#endif
#if AUDIO_SAMPLES == 32
    4,                              /* bSubFrameSize        4                   */
    32,                             /* bBitResolution       32                  */
#endif
    2,                              /* bSamFreqType         2 frequencies       */
    LE24(48000ul),                  /*                      48 kHz              */
    LE24(96000ul),                  /*                      96 kHz              */

    /* Audio streaming IN endpoint */
    USBD_SIZE_ENDPOINT,             /* bLength                                  */
    USBD_ENDPOINT,                  /* bDescriptorType                          */
    USB_ENDPOINT_IQ,                /* bEndpointAddress     Logical EP 3 IN     */
    0x05,                           /* bmAttributes         isochronous, asynch */
                                    /*                      data                */
    LE16(USB_ISOC_SIZE_IQ),         /* wMaxPacketSize       (init.h)            */
    1,                              /* bInterval                                */
    0,                              /* bRefresh                                 */
    0,                              /* bSyncAddress                             */

    /* Class specific audio endpoint */
    7,
    USBD_CS_ENDPOINT,
    1,                              /* bDescriptorSubtype   EP_GENERAL          */
    1,                              /* bmAttributes         Sample rate control */
    0,                              /* bLockDelayUnits      1:ms, 2:Samples     */
    LE16(0),                        /* wLockDelay                               */

    /* --- Soundcard Demodulator --- */

    /* Standard AudioControl Interface */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOC_DEMOD,     /* bInterfaceNumber                         */
    0,                              /* bAlternateSetting    0                   */
    0,                              /* bNumEndpoints        0                   */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOCONTROL,       /* bInterfaceSubClass   AUDIOCONTROL        */
    0,                              /* bInterfaceProtocol                       */
    11,                             /* iInterface           "FiFiSDR Demodulator" */

    /* Class-Specific AC Interface Header */
    USBD_SIZE_AUDIO_ST_HEADER(1),
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_HEADER,           /* bDescriptorSubtype   HEADER              */
    LE16(0x0100),                   /* bcdADC               1.0                 */
    LE16(                           /* wTotalLength                             */
        USBD_SIZE_AUDIO_ST_HEADER(1) +
        USBD_SIZE_AUDIO_ST_INPUT_TERMINAL +
        USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL +
        USBD_SIZE_AUDIO_ST_FEATURE_UNIT(1,1) +
        0
      ),
    1,                              /* bInCollection        1                   */
    USB_INTERFACE_AUDIOS_DEMOD,     /* baInterfaceNr(1)                         */

    /* Input Terminal */
    USBD_SIZE_AUDIO_ST_INPUT_TERMINAL,
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_INPUT_TERMINAL,   /* bDescriptorSubtype   INPUT_TERMINAL      */
    USB_UNITID_DEMOD_IN,            /* bTerminalID                              */
    LE16(0x0201),                   /* wTerminalType        LineIn              */
    0,                              /* bAssocTerminal       no association      */
    1,                              /* bNrChannels          1                   */
    LE16(0x0001),                   /* wChannelConfig       Front L             */
    0,                              /* iChannelNames        (none)              */
    12,                             /* iTerminal            "Demodulator"       */

    /* Output Terminal */
    USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL,
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_OUTPUT_TERMINAL,  /* bDescriptorSubtype   OUTPUT_TERMINAL     */
    USB_UNITID_DEMOD_OUTPUT,        /* bTerminalID                              */
    LE16(0x0101),                   /* wTerminalType        USB Streaming       */
    0,                              /* bAssocTerminal       no association      */
    USB_UNITID_DEMOD_FEATURE,       /* bSourceID            (Feature)           */
    0,                              /* iTerminal            (none)              */

    /* Feature Unit */
    USBD_SIZE_AUDIO_ST_FEATURE_UNIT(1,1),
    USBD_CS_INTERFACE,
    USBD_AUDIO_ST_FEATURE_UNIT,     /* bDescriptorSubtype   FEATURE_UNIT        */
    USB_UNITID_DEMOD_FEATURE,       /* bUnitID                                  */
    USB_UNITID_DEMOD_IN,            /* bSourceID            (Demodulator)       */
    1,                              /* bControlSize         n = 1 Byte/channel  */
    0x03,                           /* bmaControls(0)       Master: Mute+Volume */
    0x00,                           /* bmaControls(0)                           */
0,//    16,                             /* iFeature             "Audio"             */

    /* Standard AudioStreaming Interface (alternate setting #0) */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOS_DEMOD,     /* bInterfaceNumber                         */
    0,                              /* bAlternateSetting    0                   */
    0,                              /* bNumEndPoints        no endpoints        */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOSTREAMING,     /* bInterfaceSubClass   AUDIOSTREAMING      */
    0,                              /* bInterfaceProtocol                       */
    5,                              /* iInterface           "UDA1361 Capture"   */

    /* Standard AudioStreaming Interface (alternate setting #1) */
    USBD_SIZE_INTERFACE,            /* bLength                                  */
    USBD_INTERFACE,                 /* bDescriptorType                          */
    USB_INTERFACE_AUDIOS_DEMOD,     /* bInterfaceNumber                         */
    1,                              /* bAlternateSetting    1                   */
    1,                              /* bNumEndPoints        1 data endpoint     */
    USBC_AUDIO,                     /* bInterfaceClass      AUDIO               */
    USBCS_AUDIO_AUDIOSTREAMING,     /* bInterfaceSubClass   AUDIOSTREAMING      */
    0,                              /* bInterfaceProtocol                       */
    5,                              /* iInterface           "UDA1361 Capture"   */

    /* Audio Stream Audio Class */
    7,
    USBD_CS_INTERFACE,
    1,                              /* bDescriptorSubtype   AS_GENERAL,         */
    USB_UNITID_DEMOD_OUTPUT,        /* bTerminalLink        (Demod out term)    */
    1,                              /* bDelay               1                   */
    LE16(0x0001),                   /* wFormatTag           PCM                 */

    /* Format Type Audio */
    11,
    USBD_CS_INTERFACE,
    2,                              /* bDescriptorSubtype   FORMAT_TYPE         */
    1,                              /* bFormatType          TYPE_I              */
    1,                              /* bNrChannels          1                   */
    2,                              /* bSubFrameSize        2                   */
    16,                             /* bBitResolution       16                  */
    1,                              /* bSamFreqType         1 frequency         */
    LE24(8000ul),                   /*                      8 kHz               */

    /* Audio streaming IN endpoint */
    USBD_SIZE_ENDPOINT,             /* bLength                                  */
    USBD_ENDPOINT,                  /* bDescriptorType                          */
    USB_ENDPOINT_DEMOD,             /* bEndpointAddress     Logical EP 6 IN     */
    0x05,                           /* bmAttributes         isochronous, asynch */
                                    /*                      data                */
    LE16(USB_ISOC_SIZE_DEMOD),      /* wMaxPacketSize       (init.h)            */
    1,                              /* bInterval                                */
    0,                              /* bRefresh                                 */
    0,                              /* bSyncAddress                             */

    /* Class specific audio endpoint */
    7,
    USBD_CS_ENDPOINT,
    1,                              /* bDescriptorSubtype   EP_GENERAL          */
    1,                              /* bmAttributes         Sample rate control */
    0,                              /* bLockDelayUnits      1:ms, 2:Samples     */
    LE16(0),                        /* wLockDelay                               */

    /* --- String Descriptors --- */

    /* String 0 contains the language codes */
    4, USBD_STRING,
    LE16(0x0409),                   /* English                                  */

#if 1
    /* #1: Device idVendor */
    42, USBD_STRING,
    'w', 0,
    'w', 0,
    'w', 0,
    '.', 0,
    'o', 0,
    'v', 0,
    '-', 0,
    'l', 0,
    'e', 0,
    'n', 0,
    'n', 0,
    'e', 0,
    's', 0,
    't', 0,
    'a', 0,
    'd', 0,
    't', 0,
    '.', 0,
    'd', 0,
    'e', 0,

    /* #2: Device idProduct */
    18, USBD_STRING,
    'F', 0,
    'i', 0,
    'F', 0,
    'i', 0,
    '-', 0,
    'S', 0,
    'D', 0,
    'R', 0,

    /* #3: Device idSerial. Only a dummy! */
    4, USBD_STRING,
    '1', 0,
#else
    /* #1: Device idVendor */
    26, USBD_STRING,
    'w', 0,
    'w', 0,
    'w', 0,
    '.', 0,
    'o', 0,
    'b', 0,
    'd', 0,
    'e', 0,
    'v', 0,
    '.', 0,
    'a', 0,
    't', 0,

    /* #2: Device idProduct */
    22, USBD_STRING,
    'D', 0,
    'G', 0,
    '8', 0,
    'S', 0,
    'A', 0,
    'Q', 0,
    '-', 0,
    'I', 0,
    '2', 0,
    'C', 0,

    /* #3: Device idSerial. */
    22, USBD_STRING,
    'P', 0,
    'E', 0,
    '0', 0,
    'F', 0,
    'K', 0,
    'O', 0,
    '-', 0,
    '2', 0,
    '.', 0,
    '0', 0,
#endif

    /* #4:  */
    38, USBD_STRING,
    'F', 0,
    'i', 0,
    'F', 0,
    'i', 0,
    '-', 0,
    'S', 0,
    'D', 0,
    'R', 0,
    ' ', 0,
    'S', 0,
    'o', 0,
    'u', 0,
    'n', 0,
    'd', 0,
    'c', 0,
    'a', 0,
    'r', 0,
    'd', 0,

    /* #5:  */
    32, USBD_STRING,
    'U', 0,
    'D', 0,
    'A', 0,
    '1', 0,
    '3', 0,
    '6', 0,
    '1', 0,
    ' ', 0,
    'E', 0,
    'i', 0,
    'n', 0,
    'g', 0,
    'a', 0,
    'n', 0,
    'g', 0,

    /* #6:  */
    26, USBD_STRING,
    'S', 0,
    'o', 0,
    'f', 0,
    't', 0,
    'R', 0,
    'o', 0,
    'c', 0,
    'k', 0,
    ' ', 0,
    'S', 0,
    'D', 0,
    'R', 0,

    /* #7:  */
    4, USBD_STRING,
    'I', 0,

    /* #8:  */
    4, USBD_STRING,
    'Q', 0,

    /* #9:  */
    14, USBD_STRING,
    'P', 0,
    'r', 0,
    'e', 0,
    'a', 0,
    'm', 0,
    'p', 0,

    /* #10:  */
    36, USBD_STRING,
    'F', 0,
    'i', 0,
    'F', 0,
    'i', 0,
    'S', 0,
    'D', 0,
    'R', 0,
    ' ', 0,
    'S', 0,
    'o', 0,
    'u', 0,
    'n', 0,
    'd', 0,
    'c', 0,
    'a', 0,
    'r', 0,
    'd', 0,

    /* #11:  */
    40, USBD_STRING,
    'F', 0,
    'i', 0,
    'F', 0,
    'i', 0,
    'S', 0,
    'D', 0,
    'R', 0,
    ' ', 0,
    'D', 0,
    'e', 0,
    'm', 0,
    'o', 0,
    'd', 0,
    'u', 0,
    'l', 0,
    'a', 0,
    't', 0,
    'o', 0,
    'r', 0,

    /* #12:  */
    12, USBD_STRING,
    'D', 0,
    'e', 0,
    'm', 0,
    'o', 0,
    'd', 0,

    /* #13:  */
    8, USBD_STRING,
    '-', 0,
    '-', 0,
    '-', 0,

    /* #14:  */
    6, USBD_STRING,
    '-', 0,
    '-', 0,

    /* #15:  */
    10, USBD_STRING,
    'M', 0,
    'o', 0,
    'd', 0,
    'e', 0,

    /* #16:  */
    12, USBD_STRING,
    'A', 0,
    'u', 0,
    'd', 0,
    'i', 0,
    'o', 0,

    /* End of descriptors */
    0
};

